# The contents of this file are subject to the Mozilla Public
# License Version 1.1 (the "License"); you may not use this file
# except in compliance with the License. You may obtain a copy of
# the License at http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS
# IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
# implied. See the License for the specific language governing
# rights and limitations under the License.
#
# The Original Code is ChangeNotification version 1.0.
#
# The Initial Owner of the Original Code is European Environment
# Agency (EEA).  Portions created by CMG and Finsiel Romania are
# Copyright (C) European Environment Agency.  All
# Rights Reserved.
#
# Contributor(s):
# Cornel Nitu, Finsiel Romania

#This product includes software developed by Jens Vagelpohl
#for use in the Z Object Publishing Environment
#(http://www.zope.org/).


class Filter:
    """Filter for test on specific properties"""
    def __init__(self, id, expr_filter):
        """Constructor"""
        self.id = id
        self.expr_filter = expr_filter


class FilterManager:
    """Manager for Filter objects"""

    def __init__(self):
        """Constructor"""
        self.__filters = {}

    def __createFilter(self, id, expr_filter):
        """Add a new filter to filters dictionary"""
        filter = Filter(id, expr_filter)
        self.__filters[id] = filter

    def __modifyFilter(self, id, expr_filter):
        """Modify filter"""
        try:
            filter = self.__filters[id]
        except:
            pass
        else:
            filter.expr_filter = expr_filter

    def __deleteFilter(self, id):
        """Delete a filter"""
        try:
            del(self.__filters[id])
        except:
            pass

    def getFiltersList(self):
        """Get a list with all filters"""
        try:
            return self.__filters.values()
        except:
            return None

    def getFilter(self, id):
        """Get a filter object"""
        try:
            return self.__filters[id]
        except:
            return None

    def createFilter(self, id, expr_filter):
        """Create a new filter"""
        self.__createFilter(id, expr_filter)
        self._p_changed = 1

    def modifyFilter(self, id, expr_filter):
        """Modify filter's properties"""
        self.__modifyFilter(id, expr_filter)
        self._p_changed = 1

    def deleteFilter(self, ids):
        """Delete 1 or more filters"""
        for id in ids:
            self.__deleteFilter(id)
        self._p_changed = 1
